Package: ExPanDaR
Type: Package
Title: Explore Your Data Interactively
Version: 0.5.3
Authors@R: person("Joachim", "Gassen", role = c("aut", "cre"), 
    email = "gassen@wiwi.hu-berlin.de",
    comment = c(ORCID = "0000-0003-4364-2911"))
Description: Provides a shiny-based front end (the 'ExPanD' app) and
    a set of functions for exploratory data analysis. Run as a web-based 
    app, 'ExPanD' enables users to assess the robustness of empirical evidence 
    without providing them access to the underlying data. You can export a 
    notebook containing the analysis of 'ExPanD' and/or use the functions of the 
    package to support your exploratory data analysis workflow. Refer to the 
    vignettes of the package for more information on how to use 'ExPanD' and/or 
    the functions of this package.
Depends: R (>= 3.3.0)
License: MIT + file LICENSE
URL: https://joachim-gassen.github.io/ExPanDaR/
BugReports: https://github.com/joachim-gassen/ExPanDaR/issues
Encoding: UTF-8
LazyData: true
Imports: tidyr, dplyr, ggplot2, corrplot, multiwayvcov, lmtest, plm,
        stargazer, scales, shiny, DT, openssl, tictoc, shinycssloaders,
        kableExtra, rio, zip, rlang
RoxygenNote: 7.1.0
Suggests: gapminder, rmarkdown, htmltools, knitr, devtools
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2020-12-06 13:02:10 UTC; joachim
Author: Joachim Gassen [aut, cre] (<https://orcid.org/0000-0003-4364-2911>)
Maintainer: Joachim Gassen <gassen@wiwi.hu-berlin.de>
Repository: CRAN
Date/Publication: 2020-12-06 23:50:03 UTC
Built: R 4.6.0; ; 2026-04-30 11:02:12 UTC; unix
